@@ -54,13 +54,16 @@ class AbstractPyTorchTreeImpl(AbstracTreeImpl, torch.nn.Module):
     Abstract class definig the basic structure for tree-base models implemented in PyTorch.
     """
 
-    def __init__(self, logical_operator, tree_parameters, n_features, classes, n_classes, **kwargs):
+    def __init__(
+        self, logical_operator, tree_parameters, n_features, classes, n_classes, decision_cond="<=", extra_config={}, **kwargs
+    ):
         """
         Args:
             tree_parameters: The parameters defining the tree structure
             n_features: The number of features input to the model
             classes: The classes used for classification. None if implementing a regression model
             n_classes: The total number of used classes
+            decision_cond: The condition of the decision nodes in the x <cond> threshold order. Default '<='. Values can be <=, <, >=, >
         """
         super(AbstractPyTorchTreeImpl, self).__init__(logical_operator, **kwargs)
 
@@ -84,6 +87,29 @@ class AbstractPyTorchTreeImpl(AbstracTreeImpl, torch.nn.Module):
                 self.classes = torch.nn.Parameter(torch.IntTensor(classes), requires_grad=False)
                 self.perform_class_select = True
 
+        # Set the decision condition.
+        decision_cond_map = {"<=": torch.le, "<": torch.lt, ">=": torch.ge, ">": torch.gt}
+        assert decision_cond in decision_cond_map.keys(), "decision_cond has to be one of:{}".format(
+            ",".join(decision_cond_map.keys())
+        )
+        self.decision_cond = decision_cond_map[decision_cond]
+
+        # In some cases float64 is required oterwise we will lose precision.
+        tree_op_precision_dtype = None
+        if constants.TREE_OP_PRECISION_DTYPE in extra_config:
+            tree_op_precision_dtype = extra_config[constants.TREE_OP_PRECISION_DTYPE]
+            assert tree_op_precision_dtype in ["float32", "float64"], "{} has to be of type float32 or float64".format(
+                constants.TREE_OP_PRECISION_DTYPE
+            )
+        else:
+            tree_op_precision_dtype = "float32"
+        self.tree_op_precision_dtype = tree_op_precision_dtype
+
+        # We register also base_prediction here so that tensor will be moved to the proper hardware with the model.
+        # i.e., if cuda is selected, the parameter will be automatically moved on the GPU.
+        if constants.BASE_PREDICTION in extra_config:
+            self.base_prediction = extra_config[constants.BASE_PREDICTION]
+
 
 class GEMMTreeImpl(AbstractPyTorchTreeImpl):
     """
@@ -100,7 +126,9 @@ class GEMMTreeImpl(AbstractPyTorchTreeImpl):
         """
         # If n_classes is not provided we induce it from tree parameters. Multioutput regression targets are also treated as separate classes.
         n_classes = n_classes if n_classes is not None else tree_parameters[0][0][2].shape[0]
-        super(GEMMTreeImpl, self).__init__(logical_operator, tree_parameters, n_features, classes, n_classes, **kwargs)
+        super(GEMMTreeImpl, self).__init__(
+            logical_operator, tree_parameters, n_features, classes, n_classes, extra_config=extra_config, **kwargs
+        )
 
         # Initialize the actual model.
         hidden_one_size = 0
@@ -113,10 +141,10 @@ class GEMMTreeImpl(AbstractPyTorchTreeImpl):
 
         n_trees = len(tree_parameters)
         weight_1 = np.zeros((n_trees, hidden_one_size, n_features))
-        bias_1 = np.zeros((n_trees, hidden_one_size))
+        bias_1 = np.zeros((n_trees, hidden_one_size), dtype=np.float64)
         weight_2 = np.zeros((n_trees, hidden_two_size, hidden_one_size))
         bias_2 = np.zeros((n_trees, hidden_two_size))
-        weight_3 = np.zeros((n_trees, hidden_three_size, hidden_two_size))
+        weight_3 = np.zeros((n_trees, hidden_three_size, hidden_two_size), dtype=np.float64)
 
         for i, (weight, bias) in enumerate(tree_parameters):
             if len(weight[0]) > 0:
@@ -133,24 +161,19 @@ class GEMMTreeImpl(AbstractPyTorchTreeImpl):
         self.hidden_three_size = hidden_three_size
 
         self.weight_1 = torch.nn.Parameter(torch.from_numpy(weight_1.reshape(-1, self.n_features).astype("float32")))
-        self.bias_1 = torch.nn.Parameter(torch.from_numpy(bias_1.reshape(-1, 1).astype("float32")))
+        self.bias_1 = torch.nn.Parameter(torch.from_numpy(bias_1.reshape(-1, 1).astype(self.tree_op_precision_dtype)))
 
         self.weight_2 = torch.nn.Parameter(torch.from_numpy(weight_2.astype("float32")))
         self.bias_2 = torch.nn.Parameter(torch.from_numpy(bias_2.reshape(-1, 1).astype("float32")))
 
-        self.weight_3 = torch.nn.Parameter(torch.from_numpy(weight_3.astype("float32")))
-
-        # We register also base_prediction here so that tensor will be moved to the proper hardware with the model.
-        # i.e., if cuda is selected, the parameter will be automatically moved on the GPU.
-        if constants.BASE_PREDICTION in extra_config:
-            self.base_prediction = extra_config[constants.BASE_PREDICTION]
+        self.weight_3 = torch.nn.Parameter(torch.from_numpy(weight_3.astype(self.tree_op_precision_dtype)))
 
     def aggregation(self, x):
         return x
 
     def forward(self, x):
         x = x.t()
-        x = torch.mm(self.weight_1, x) < self.bias_1
+        x = self.decision_cond(torch.mm(self.weight_1, x), self.bias_1)
         x = x.view(self.n_trees, self.hidden_one_size, -1)
         x = x.float()
 
@@ -158,7 +181,10 @@ class GEMMTreeImpl(AbstractPyTorchTreeImpl):
 
         x = x.view(self.n_trees * self.hidden_two_size, -1) == self.bias_2
         x = x.view(self.n_trees, self.hidden_two_size, -1)
-        x = x.float()
+        if self.tree_op_precision_dtype == "float32":
+            x = x.float()
+        else:
+            x = x.double()
 
         x = torch.matmul(self.weight_3, x)
         x = x.view(self.n_trees, self.hidden_three_size, -1)
@@ -203,7 +229,7 @@ class TreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
         # If n_classes is not provided we induce it from tree parameters. Multioutput regression targets are also treated as separate classes.
         n_classes = n_classes if n_classes is not None else tree_parameters[0][6].shape[1]
         super(TreeTraversalTreeImpl, self).__init__(
-            logical_operator, tree_parameters, n_features, classes, n_classes, **kwargs
+            logical_operator, tree_parameters, n_features, classes, n_classes, extra_config=extra_config, **kwargs
         )
 
         # Initialize the actual model.
@@ -216,8 +242,8 @@ class TreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
         rights = np.zeros((self.num_trees, self.num_nodes), dtype=np.int64)
 
         features = np.zeros((self.num_trees, self.num_nodes), dtype=np.int64)
-        thresholds = np.zeros((self.num_trees, self.num_nodes), dtype=np.float32)
-        values = np.zeros((self.num_trees, self.num_nodes, self.n_classes), dtype=np.float32)
+        thresholds = np.zeros((self.num_trees, self.num_nodes), dtype=np.float64)
+        values = np.zeros((self.num_trees, self.num_nodes, self.n_classes), dtype=np.float64)
 
         for i in range(self.num_trees):
             lefts[i][: len(tree_parameters[i][0])] = tree_parameters[i][2]
@@ -230,17 +256,14 @@ class TreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
         self.rights = torch.nn.Parameter(torch.from_numpy(rights).view(-1), requires_grad=False)
 
         self.features = torch.nn.Parameter(torch.from_numpy(features).view(-1), requires_grad=False)
-        self.thresholds = torch.nn.Parameter(torch.from_numpy(thresholds).view(-1))
-        self.values = torch.nn.Parameter(torch.from_numpy(values).view(-1, self.n_classes))
+        self.thresholds = torch.nn.Parameter(torch.from_numpy(thresholds.astype(self.tree_op_precision_dtype)).view(-1))
+        self.values = torch.nn.Parameter(
+            torch.from_numpy(values.astype(self.tree_op_precision_dtype)).view(-1, self.n_classes)
+        )
 
         nodes_offset = [[i * self.num_nodes for i in range(self.num_trees)]]
         self.nodes_offset = torch.nn.Parameter(torch.LongTensor(nodes_offset), requires_grad=False)
 
-        # We register also base_prediction here so that tensor will be moved to the proper hardware with the model.
-        # i.e., if cuda is selected, the parameter will be automatically moved on the GPU.
-        if constants.BASE_PREDICTION in extra_config:
-            self.base_prediction = extra_config[constants.BASE_PREDICTION]
-
     def aggregation(self, x):
         return x
 
@@ -256,7 +279,7 @@ class TreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
             lefts = torch.index_select(self.lefts, 0, indexes).view(-1, self.num_trees)
             rights = torch.index_select(self.rights, 0, indexes).view(-1, self.num_trees)
 
-            indexes = torch.where(torch.ge(feature_values, thresholds), rights, lefts).long()
+            indexes = torch.where(self.decision_cond(feature_values, thresholds), lefts, rights).long()
             indexes = indexes + self.nodes_offset
             indexes = indexes.view(-1)
 
@@ -296,7 +319,7 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
         # If n_classes is not provided we induce it from tree parameters. Multioutput regression targets are also treated as separate classes.
         n_classes = n_classes if n_classes is not None else tree_parameters[0][6].shape[1]
         super(PerfectTreeTraversalTreeImpl, self).__init__(
-            logical_operator, tree_parameters, n_features, classes, n_classes, **kwargs
+            logical_operator, tree_parameters, n_features, classes, n_classes, extra_config=extra_config, **kwargs
         )
 
         # Initialize the actual model.
@@ -307,7 +330,7 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
         node_maps = [tp[0] for tp in tree_parameters]
 
         weight_0 = np.zeros((self.num_trees, 2 ** max_depth - 1))
-        bias_0 = np.zeros((self.num_trees, 2 ** max_depth - 1))
+        bias_0 = np.zeros((self.num_trees, 2 ** max_depth - 1), dtype=np.float64)
         weight_1 = np.zeros((self.num_trees, 2 ** max_depth, self.n_classes))
 
         for i, node_map in enumerate(node_maps):
@@ -315,9 +338,10 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
 
         node_by_levels = [set() for _ in range(max_depth)]
         self._traverse_by_level(node_by_levels, 0, -1, max_depth)
-
         self.root_nodes = torch.nn.Parameter(torch.from_numpy(weight_0[:, 0].flatten().astype("int64")), requires_grad=False)
-        self.root_biases = torch.nn.Parameter(-1 * torch.from_numpy(bias_0[:, 0].astype("float32")), requires_grad=False)
+        self.root_biases = torch.nn.Parameter(
+            torch.from_numpy(bias_0[:, 0].astype(self.tree_op_precision_dtype)), requires_grad=False
+        )
 
         tree_indices = np.array([i for i in range(0, 2 * self.num_trees, 2)]).astype("int64")
         self.tree_indices = torch.nn.Parameter(torch.from_numpy(tree_indices), requires_grad=False)
@@ -329,7 +353,7 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
                 torch.from_numpy(weight_0[:, list(sorted(node_by_levels[i]))].flatten().astype("int64")), requires_grad=False
             )
             biases = torch.nn.Parameter(
-                torch.from_numpy(-1 * bias_0[:, list(sorted(node_by_levels[i]))].flatten().astype("float32")),
+                torch.from_numpy(bias_0[:, list(sorted(node_by_levels[i]))].flatten().astype(self.tree_op_precision_dtype)),
                 requires_grad=False,
             )
             self.nodes.append(nodes)
@@ -339,19 +363,14 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
         self.biases = torch.nn.ParameterList(self.biases)
 
         self.leaf_nodes = torch.nn.Parameter(
-            torch.from_numpy(weight_1.reshape((-1, self.n_classes)).astype("float32")), requires_grad=False
+            torch.from_numpy(weight_1.reshape((-1, self.n_classes)).astype(self.tree_op_precision_dtype)), requires_grad=False
         )
 
-        # We register also base_prediction here so that tensor will be moved to the proper hardware with the model.
-        # i.e., if cuda is selected, the parameter will be automatically moved on the GPU.
-        if constants.BASE_PREDICTION in extra_config:
-            self.base_prediction = extra_config[constants.BASE_PREDICTION]
-
     def aggregation(self, x):
         return x
 
     def forward(self, x):
-        prev_indices = (torch.ge(torch.index_select(x, 1, self.root_nodes), self.root_biases)).long()
+        prev_indices = (self.decision_cond(torch.index_select(x, 1, self.root_nodes), self.root_biases)).long()
         prev_indices = prev_indices + self.tree_indices
         prev_indices = prev_indices.view(-1)
 
@@ -359,7 +378,9 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
         for nodes, biases in zip(self.nodes, self.biases):
             gather_indices = torch.index_select(nodes, 0, prev_indices).view(-1, self.num_trees)
             features = torch.gather(x, 1, gather_indices).view(-1)
-            prev_indices = factor * prev_indices + torch.ge(features, torch.index_select(biases, 0, prev_indices)).long()
+            prev_indices = (
+                factor * prev_indices + self.decision_cond(features, torch.index_select(biases, 0, prev_indices)).long()
+            )
 
         output = torch.index_select(self.leaf_nodes, 0, prev_indices).view(-1, self.num_trees, self.n_classes)
 
@@ -390,20 +411,11 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
     def _get_weights_and_biases(self, nodes_map, tree_depth, weight_0, weight_1, bias_0):
         def depth_f_traversal(node, current_depth, node_id, leaf_start_id):
             weight_0[node_id] = node.feature
-            bias_0[node_id] = -node.threshold
+            bias_0[node_id] = node.threshold
             current_depth += 1
             node_id += 1
 
-            if node.left.feature == -1:
-                node_id += 2 ** (tree_depth - current_depth - 1) - 1
-                v = node.left.value
-                weight_1[leaf_start_id : leaf_start_id + 2 ** (tree_depth - current_depth - 1)] = (
-                    np.ones((2 ** (tree_depth - current_depth - 1), self.n_classes)) * v
-                )
-                leaf_start_id += 2 ** (tree_depth - current_depth - 1)
-            else:
-                node_id, leaf_start_id = depth_f_traversal(node.left, current_depth, node_id, leaf_start_id)
-
+            # Condition false (right sub-tree)
             if node.right.feature == -1:
                 node_id += 2 ** (tree_depth - current_depth - 1) - 1
                 v = node.right.value
@@ -414,6 +426,17 @@ class PerfectTreeTraversalTreeImpl(AbstractPyTorchTreeImpl):
             else:
                 node_id, leaf_start_id = depth_f_traversal(node.right, current_depth, node_id, leaf_start_id)
 
+            # Condition true (left sub-tree)
+            if node.left.feature == -1:
+                node_id += 2 ** (tree_depth - current_depth - 1) - 1
+                v = node.left.value
+                weight_1[leaf_start_id : leaf_start_id + 2 ** (tree_depth - current_depth - 1)] = (
+                    np.ones((2 ** (tree_depth - current_depth - 1), self.n_classes)) * v
+                )
+                leaf_start_id += 2 ** (tree_depth - current_depth - 1)
+            else:
+                node_id, leaf_start_id = depth_f_traversal(node.left, current_depth, node_id, leaf_start_id)
+
             return node_id, leaf_start_id
 
         depth_f_traversal(nodes_map[0], -1, 0, 0)
@@ -426,14 +449,17 @@ class GEMMDecisionTreeImpl(GEMMTreeImpl):
 
     """
 
-    def __init__(self, logical_operator, tree_parameters, n_features, classes=None):
+    def __init__(self, logical_operator, tree_parameters, n_features, classes=None, extra_config={}):
         """
         Args:
             tree_parameters: The parameters defining the tree structure
             n_features: The number of features input to the model
             classes: The classes used for classification. None if implementing a regression model
+            extra_config: Extra configuration used to properly implement the source tree
         """
-        super(GEMMDecisionTreeImpl, self).__init__(logical_operator, tree_parameters, n_features, classes)
+        super(GEMMDecisionTreeImpl, self).__init__(
+            logical_operator, tree_parameters, n_features, classes, extra_config=extra_config
+        )
 
     def aggregation(self, x):
         output = x.sum(0).t()
@@ -446,7 +472,7 @@ class TreeTraversalDecisionTreeImpl(TreeTraversalTreeImpl):
     Class implementing the Tree Traversal strategy in PyTorch for decision tree models.
     """
 
-    def __init__(self, logical_operator, tree_parameters, max_depth, n_features, classes=None, extra_config={}):
+    def __init__(self, logical_operator, tree_parameters, max_depth, n_features, classes=None, extra_config={}, **kwargs):
         """
         Args:
             tree_parameters: The parameters defining the tree structure
@@ -456,7 +482,7 @@ class TreeTraversalDecisionTreeImpl(TreeTraversalTreeImpl):
             extra_config: Extra configuration used to properly implement the source tree
         """
         super(TreeTraversalDecisionTreeImpl, self).__init__(
-            logical_operator, tree_parameters, max_depth, n_features, classes, extra_config=extra_config
+            logical_operator, tree_parameters, max_depth, n_features, classes, extra_config=extra_config, **kwargs
         )
 
     def aggregation(self, x):
@@ -470,16 +496,17 @@ class PerfectTreeTraversalDecisionTreeImpl(PerfectTreeTraversalTreeImpl):
     Class implementing the Perfect Tree Traversal strategy in PyTorch for decision tree models.
     """
 
-    def __init__(self, logical_operator, tree_parameters, max_depth, n_features, classes=None):
+    def __init__(self, logical_operator, tree_parameters, max_depth, n_features, classes=None, extra_config={}, **kwargs):
         """
         Args:
             tree_parameters: The parameters defining the tree structure
             max_depth: The maximum tree-depth in the model
             n_features: The number of features input to the model
             classes: The classes used for classification. None if implementing a regression model
+            extra_config: Extra configuration used to properly implement the source tree
         """
         super(PerfectTreeTraversalDecisionTreeImpl, self).__init__(
-            logical_operator, tree_parameters, max_depth, n_features, classes
+            logical_operator, tree_parameters, max_depth, n_features, classes, extra_config=extra_config, **kwargs
         )
 
     def aggregation(self, x):
@@ -494,7 +521,7 @@ class GEMMGBDTImpl(GEMMTreeImpl):
     Class implementing the GEMM strategy (in PyTorch) for GBDT models.
     """
 
-    def __init__(self, logical_operator, tree_parameters, n_features, classes=None, extra_config={}):
+    def __init__(self, logical_operator, tree_parameters, n_features, classes=None, extra_config={}, **kwargs):
         """
         Args:
             tree_parameters: The parameters defining the tree structure
@@ -502,7 +529,7 @@ class GEMMGBDTImpl(GEMMTreeImpl):
             classes: The classes used for classification. None if implementing a regression model
             extra_config: Extra configuration used to properly implement the source tree
         """
-        super(GEMMGBDTImpl, self).__init__(logical_operator, tree_parameters, n_features, classes, 1, extra_config)
+        super(GEMMGBDTImpl, self).__init__(logical_operator, tree_parameters, n_features, classes, 1, extra_config, **kwargs)
 
         self.n_gbdt_classes = 1
         self.post_transform = _tree_commons.PostTransform()
@@ -526,7 +553,7 @@ class TreeTraversalGBDTImpl(TreeTraversalTreeImpl):
     Class implementing the Tree Traversal strategy in PyTorch.
     """
 
-    def __init__(self, logical_operator, tree_parameters, max_detph, n_features, classes=None, extra_config={}):
+    def __init__(self, logical_operator, tree_parameters, max_detph, n_features, classes=None, extra_config={}, **kwargs):
         """
         Args:
             tree_parameters: The parameters defining the tree structure
@@ -536,7 +563,7 @@ class TreeTraversalGBDTImpl(TreeTraversalTreeImpl):
             extra_config: Extra configuration used to properly implement the source tree
         """
         super(TreeTraversalGBDTImpl, self).__init__(
-            logical_operator, tree_parameters, max_detph, n_features, classes, 1, extra_config
+            logical_operator, tree_parameters, max_detph, n_features, classes, 1, extra_config, **kwargs
         )
 
         self.n_gbdt_classes = 1
@@ -561,7 +588,7 @@ class PerfectTreeTraversalGBDTImpl(PerfectTreeTraversalTreeImpl):
     Class implementing the Perfect Tree Traversal strategy in PyTorch.
     """
 
-    def __init__(self, logical_operator, tree_parameters, max_depth, n_features, classes=None, extra_config={}):
+    def __init__(self, logical_operator, tree_parameters, max_depth, n_features, classes=None, extra_config={}, **kwargs):
         """
         Args:
             tree_parameters: The parameters defining the tree structure
@@ -571,7 +598,7 @@ class PerfectTreeTraversalGBDTImpl(PerfectTreeTraversalTreeImpl):
             extra_config: Extra configuration used to properly implement the source tree
         """
         super(PerfectTreeTraversalGBDTImpl, self).__init__(
-            logical_operator, tree_parameters, max_depth, n_features, classes, 1, extra_config
+            logical_operator, tree_parameters, max_depth, n_features, classes, 1, extra_config, **kwargs
         )
 
         self.n_gbdt_classes = 1
